package ch.phoenix.oss.quarkus.commons.tracing;

import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.ArgumentMatchers.startsWith;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.verifyNoMoreInteractions;

import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.junit.mockito.InjectSpy;
import io.restassured.RestAssured;
import io.restassured.http.ContentType;
import java.util.Map;
import org.junit.jupiter.api.Test;

@QuarkusTest
class RoutePatternTest {

    @InjectSpy
    TracingService tracingService;

    @Test
    void getBlankResource() {
        var route = "/";
        RestAssured.given().accept(ContentType.TEXT).when().get(route).then().statusCode(200);

        verifyGetTracing(route, Map.of());
    }

    @Test
    void postBlankResource() {
        var route = "/";
        RestAssured.given().accept(ContentType.TEXT).when().post(route).then().statusCode(200);

        verify(tracingService).trace("actor", "anonymous");
        verify(tracingService).trace("request.method", "POST");
        verify(tracingService).trace("request.route", route);
        verify(tracingService).trace("request.headers.accept", "text/plain");
        verify(tracingService).trace("request.headers.accept-encoding", "gzip,deflate");
        verify(tracingService).trace("request.headers.connection", "Keep-Alive");
        verify(tracingService).trace("request.headers.content-length", "0");
        verify(tracingService)
                .trace("request.headers.content-type", "application/x-www-form-urlencoded; charset=ISO-8859-1");
        verify(tracingService).trace(eq("request.headers.host"), startsWith("localhost:"));
        verify(tracingService).trace(eq("request.headers.user-agent"), startsWith("Apache-HttpClient"));
        verify(tracingService).trace("request.client.ip", "127.0.0.1");
        verifyNoMoreInteractions(tracingService);
    }

    @Test
    void getLeadingResource() {
        var route = "/leading/{id}/{anotherId}";
        RestAssured.given()
                .accept(ContentType.TEXT)
                .when()
                .get(route, 1, 2)
                .then()
                .statusCode(200);

        verifyGetTracing(route, Map.of("id", "1", "anotherId", "2"));
    }

    @Test
    void getTrailingResource() {
        var route = "/{id}/{anotherId}/trailing";
        RestAssured.given()
                .accept(ContentType.TEXT)
                .when()
                .get(route, 1, 2)
                .then()
                .statusCode(200);

        verifyGetTracing(route, Map.of("id", "1", "anotherId", "2"));
    }

    @Test
    void getLeadingAndNoTrailingResource() {
        var route = "/leading-and-no-trailing";
        RestAssured.given().accept(ContentType.TEXT).when().get(route).then().statusCode(200);

        verifyGetTracing(route, Map.of());
    }

    @Test
    void getLeadingAndNoTrailingWithSingleParamResource() {
        var route = "/leading-and-no-trailing/{param}";
        RestAssured.given().accept(ContentType.TEXT).when().get(route, 1).then().statusCode(200);

        verifyGetTracing(route, Map.of("param", "1"));
    }

    @Test
    void getLeadingAndNoTrailingWithMultiParamResource() {
        var route = "/leading-and-no-trailing/{param}/{param2}";
        RestAssured.given()
                .accept(ContentType.TEXT)
                .when()
                .get(route, 1, 2)
                .then()
                .statusCode(200);

        verifyGetTracing(route, Map.of("param", "1", "param2", "2"));
    }

    @Test
    void getLeadingAndTrailingResource() {
        var route = "/leading-and-trailing";
        RestAssured.given().accept(ContentType.TEXT).when().get(route).then().statusCode(200);

        verifyGetTracing(route, Map.of());
    }

    @Test
    void getLeadingAndTrailingWithSingleParamResource() {
        var route = "/leading-and-trailing/{param}";
        RestAssured.given().accept(ContentType.TEXT).when().get(route, 1).then().statusCode(200);

        verifyGetTracing(route, Map.of("param", "1"));
    }

    @Test
    void getLeadingAndTrailingWithMultiParamResource() {
        var route = "/leading-and-trailing/{param}/{param2}";
        RestAssured.given()
                .accept(ContentType.TEXT)
                .when()
                .get(route, 1, 2)
                .then()
                .statusCode(200);

        verifyGetTracing(route, Map.of("param", "1", "param2", "2"));
    }

    @Test
    void getNoLeadingAndNoTrailingResource() {
        var route = "/no-leading-and-no-trailing";
        RestAssured.given().accept(ContentType.TEXT).when().get(route).then().statusCode(200);

        verifyGetTracing(route, Map.of());
    }

    @Test
    void geNoLeadingAndNoTrailingWithSingleParamResource() {
        var route = "/no-leading-and-no-trailing/{param}";
        RestAssured.given().accept(ContentType.TEXT).when().get(route, 1).then().statusCode(200);

        verifyGetTracing(route, Map.of("param", "1"));
    }

    @Test
    void getNoLeadingAndNoTrailingWithMultiParamResource() {
        var route = "/no-leading-and-no-trailing/{param}/{param2}";
        RestAssured.given()
                .accept(ContentType.TEXT)
                .when()
                .get(route, 1, 2)
                .then()
                .statusCode(200);

        verifyGetTracing(route, Map.of("param", "1", "param2", "2"));
    }

    @Test
    void getNoLeadingAndTrailingResource() {
        var route = "/no-leading-and-trailing";
        RestAssured.given().accept(ContentType.TEXT).when().get(route).then().statusCode(200);

        verifyGetTracing(route, Map.of());
    }

    @Test
    void getNoLeadingAndTrailingWithSingleParamResource() {
        var route = "/no-leading-and-trailing/{param}";
        RestAssured.given().accept(ContentType.TEXT).when().get(route, 1).then().statusCode(200);

        verifyGetTracing(route, Map.of("param", "1"));
    }

    @Test
    void getNoLeadingAndTrailingWithMultiParamResource() {
        var route = "/no-leading-and-trailing/{param}/{param2}";
        RestAssured.given()
                .accept(ContentType.TEXT)
                .when()
                .get(route, 1, 2)
                .then()
                .statusCode(200);

        verifyGetTracing(route, Map.of("param", "1", "param2", "2"));
    }

    private void verifyGetTracing(String route, Map<String, String> pathParams) {
        verify(tracingService).trace("actor", "anonymous");
        verify(tracingService).trace("request.method", "GET");
        verify(tracingService).trace("request.route", route);
        pathParams.forEach((key, value) -> verify(tracingService).trace("request.path.params." + key, value));
        verify(tracingService).trace("request.headers.accept", "text/plain");
        verify(tracingService).trace("request.headers.accept-encoding", "gzip,deflate");
        verify(tracingService).trace("request.headers.connection", "Keep-Alive");
        verify(tracingService).trace(eq("request.headers.host"), startsWith("localhost:"));
        verify(tracingService).trace(eq("request.headers.user-agent"), startsWith("Apache-HttpClient"));
        verify(tracingService).trace("request.client.ip", "127.0.0.1");
        verifyNoMoreInteractions(tracingService);
    }
}
